<?php
namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");

use Log;
use Illuminate\Http\Request;
use Config;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;


class CustomerController extends Controller {

    private $helper;
    private $shopify_api_version;

    public function __construct() {      
        
        $options = new Options();
        $options->setType(true); // Makes it private
        $options->setVersion(env('SHOPIFY_API_VER'));
        // $options->setApiKey(env('SHOPIFY_API_KEY'));
        $options->setApiPassword(env('SHOPIFY_API_PASSWORD'));
        $this->shopify_api_version = env('SHOPIFY_API_VER');

        $this->api = new BasicShopifyAPI($options);
        $this->api->setSession(new Session(env('SHOPIFY_DOMAIN')));

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');

        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }

        $this->logFiles = array(
            'customerEndpoints'   => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log-customerEndpoints.txt",
                "mode"     => ""
            )            
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if ($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                    $handler = fopen($fileName, $fileDetails['mode']);
                    fputcsv($handler, $fileDetails['columns']);
                    fclose($handler);
                    touch($fileName);
                    //Make it writeable
                    chmod($fileName, 0777);
                }
            }

        }
    }

   
    public function returnResponse($status,$message, $body,$request) {
        $response = array(
            "status"       => $status,
            "message" => $message,
            "body" => $body
        );
        echo $request->callback . '(' . json_encode($response) . ')';
        die();

    }    
    
    public function getCustomerStanding(request $data) {
        
       
        if (empty($data['data'])) {

            $message = "Endpoint parameters are missing";
            $returnData = [
                'message' => $message
            ];
            $this->returnResponse(600, "error", $returnData,$data );
        }
        
        $receivedData = json_decode($data['data']);
        
        
        if (!isset($receivedData->email) || empty($receivedData->email)) {
            
            $returnData = [
                'message' => 'Parameter missing - email'
            ];

            $this->returnResponse(600, "error", $returnData,$data );

        }

         if (!isset($receivedData->orderId) || empty($receivedData->orderId)) {
            
            $returnData = [
                'message' => 'Parameter missing - orderId'
            ];

            $this->returnResponse(600, "error", $returnData,$data );

        }                  
       
        
        $requestEmail = trim($receivedData->email);
        $currentOrderId = trim($receivedData->orderId);
        $customerAvailable = false;
        
        
        $gQuery = '{
                      customers(first:250, query:"'.$requestEmail.'") {
                        edges {
                          node {
                            email
                          }
                        }
                      }
                   }';
        
        $customerResult = $this->api->graph($gQuery);
       

        if (!isset($customerResult['body']['data']['customers']['edges'][0]) || empty($customerResult['body']['data']['customers']['edges'][0])) {
            $returnData = [
                'message' => 'No such customer found in shopify',
                "customerStanding" => null 
            ];

            $this->returnResponse(200, "success", $returnData, $data );
        }else{
            $customers = $customerResult['body']['data']['customers']['edges'];     
            
            foreach( $customers as $customer){
                if($customer['node']['email'] == $requestEmail){
                    $customerAvailable = true;
                }
            }
            if(!$customerAvailable){
                $returnData = [
                    'message' => 'No such customer found in shopify',
                    "customerStanding" => null 
                ];
                $this->returnResponse(200, "success", $returnData,$data );
            }
        }
        
        $gOQuery = '{
                      orders(first:250, query:"email:'.$requestEmail.' AND status:any") {
                        edges {
                          node {
                            id
                            email
                          }
                        }
                      }
                   }';
        
        
        $customerOrderResult = $this->api->graph($gOQuery);
       
        
        if($customerAvailable && (!isset($customerOrderResult['body']['data']['orders']['edges'][0]) || empty($customerOrderResult['body']['data']['orders']['edges'][0]))) {
            
            $returnData = [
                "customerStanding" => 'New'                           
            ];
            $this->returnResponse(200, "success", $returnData,$data );;
        }
        else {
            $orders = $customerOrderResult['body']['data']['orders']['edges'];  
            $orderDetails = [];
            foreach( $orders as $order){
                if($order['node']['email'] == $requestEmail){
                    $orderId = explode('/',$order['node']['id']);
                    $orderId = end($orderId);
                    $orderDetails[] = trim($orderId);
                }
            }
            

            if(count($orderDetails) > 1){
                $returnData = [
                    "customerStanding" => 'Returning'                           
                ];
                $this->returnResponse(200, "success", $returnData,$data );
            }    
            
            if(count($orderDetails) == 1 && in_array($currentOrderId,$orderDetails) ){
                $returnData = [
                    "customerStanding" => 'New'                           
                ];
                $this->returnResponse(200, "success", $returnData,$data );
            }else{
                $returnData = [
                    "customerStanding" => 'New'                           
                ];
                $this->returnResponse(200, "success", $returnData,$data );
            }
            
        }

    }

    
}
