<?php

namespace App\Http\Controllers;

use App\Helper\Helper;
use App\Http\Controllers\Controller;
use App\Model\HcpModel;
use Illuminate\Http\Request;

class EquelleHCPController extends Controller
{
    private $hcpObj;
    private $helperObj;
    public function __construct()
    {
        $this->hcpObj = new HcpModel();
        $this->helperObj = new Helper();
        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }

    public function setPaths()
    {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList()
    {

        $this->logPath = $this->logDirectory . date('dmY');
        if (!is_dir($this->logPath)) {

            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }
        $this->logFiles = array(
            'log' => array(
                "type" => "text",
                "filename" => $this->logPath . "/equelle-hcpForm-log.txt",
                "mode" => ""),
        );
    }

    public function setWithLogFiles()
    {

        if (!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if ($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                    $handler = fopen($fileName, $fileDetails['mode']);
                    fputcsv($handler, $fileDetails['columns']);
                    fclose($handler);
                    touch($fileName);
                    chmod($fileName, 0777);
                }
            }

        }
    }


    /**
     * HCP post request
     */
    public function EquelleHCP(Request $request) {

        $requestData = $request->input('data');
        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];
            $message = 'Failed';
            $this->helperObj->logMe("error : Invalid endpoint request", $this->logFiles['log']['filename']);
            if (!empty($request->input('callback'))) {
                return $this->returnData($request->input('callback'), '600', $message, $returnData);
            } else {
                return $this->returnData(null, '600', $message, $returnData);
            }
        }
        $receivedData = json_decode($requestData);
        // $requestKeys = ['EMail', 'Title', 'FirstName', 'LastName', 
        // 'NPINumber', 'PracticeName', 'PracticePhone','AddressLine1','City','State','Zipcode'];
        // $keysInTheRequest = [];
        // $emptyValueKeys = [];
        // $nonRequiredFields = ['EMail', 'Title', 'FirstName', 'LastName', 
        // 'NPINumber', 'PracticeName', 'PracticePhone','AddressLine1','City','State','Zipcode'];
        // foreach ($receivedData as $key => $recValue) {

        //     $keysInTheRequest[] = $key;
        //     if (empty($recValue) && !in_array($key, $nonRequiredFields)) {

        //         $emptyValueKeys[] = $key;
        //     }
        // }
        // $keyComparedStatus = array_diff($requestKeys, $keysInTheRequest);
        // if (!empty($keyComparedStatus)) {

        //     $missedParameters = implode(",", $keyComparedStatus);
        //     $message = 'Failed';
        //     $returnData = [
        //         'error' => $missedParameters . ' endpoint parameters are missing in the request',
        //     ];
        //     $this->helperObj->logMe('error : ' . $missedParameters . ' endpoint parameters are missing in the request', $this->logFiles['log']['filename']);
        //     return $this->returnData($request->input('callback'), '600', $message, $returnData);
        // }
        // if (!empty($emptyValueKeys)) {

        //     $emptyValueKeys = implode(",", $emptyValueKeys);
        //     $message = 'Failed';
        //     $returnData = [
        //         'error' => $emptyValueKeys . ' endpoint parameter values are empty in the request',
        //     ];
        //     $this->helperObj->logMe('error : ' . $emptyValueKeys . ' endpoint parameter values are empty in the request', $this->logFiles['log']['filename']);
        //     return $this->returnData($request->input('callback'), '600', $message, $returnData);
        // }
        $response = $this->hcpObj->postRequestData($requestData);
        if (strtolower(trim($response)) != 'success') {

            $message = 'Failed';
            $returnData = [
                'error' => 'An error occurred while posting the request to the API',
                'request' => $requestData,
            ];
            $this->helperObj->logMe('error : An error occurred while posting the request to the API (Email: ' . 
            $receivedData->EMail . ')', $this->logFiles['log']['filename']);
            return $this->returnData($request->input('callback'), '600', $message, $returnData);
        }
        $message = 'Success';
        $returnData = [
            'message' => 'Successfully posted the given request in the API (Email: ' . $receivedData->EMail . ')',
        ];
        $this->helperObj->logMe('Success : Successfully posted the given request in the API (Email: ' . 
        $receivedData->EMail . ')', $this->logFiles['log']['filename']);
        //Log::info($returnData);
        return $this->returnData($request->input('callback'), '200', $message, $returnData);
    }

    /**
     * Response message function
     */
    public function returnData($jsonCallback = null, $status, $message, $error)
    {
        $returnData = array(
            'status' => $status,
            'message' => $message,
            'body' => $error,

        );
        if (!empty($jsonCallback)) {

            echo $jsonCallback . '(' . json_encode($returnData) . ')';
            die();
        } else {

            return response()->json($returnData);
        }
    }
}
