<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");
use App\Model\RechargeApi;
use App\Model\IterableApi;
use App\Helper\Helper;
use Illuminate\Http\Request;

use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;


class SubscriptionController extends Controller
{
    private $rechargeObj;
    private $iterableObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $api;

    public function __construct() {

        $this->rechargeObj   = new RechargeApi();
        $this->iterableObj   = new IterableApi();
        $this->helperObj  = new Helper();
        
        $options = new Options();
        $options->setType(true); // Makes it private
        $options->setVersion(env('SHOPIFY_API_VER'));
        $options->setApiKey(env('SHOPIFY_API_KEY'));
        $options->setApiPassword(env('SHOPIFY_API_PASSWORD'));

        $this->api = new BasicShopifyAPI($options);
        $this->api->setSession(new Session(env('SHOPIFY_DOMAIN')));

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');

        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }

        $this->logFiles = array(
            'subscriptionReactivationWithCoupon'   => array(
                    "type"     => "text",
                    "filename" => $this->logPath . "/log-subscriptionReactivationWithCoupon.txt",
                    "mode"     => ""),
            'otpToSubscriptionLog'   => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log-otpToSubscription.txt",
                "mode"     => ""),
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if ($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                    $handler = fopen($fileName, $fileDetails['mode']);
                    fputcsv($handler, $fileDetails['columns']);
                    fclose($handler);
                    touch($fileName);
                    //Make it writeable
                    chmod($fileName, 0777);
                }
            }

        }
    }

    /**
     * Reactivate cancelled recharge subscription with coupon
    */
    public function reactivateSubscriptionWithCoupon() {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
            // $this->sendResponse("Failed", $message, $error);
        }

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);


        $subscriptionId = '';
        $customerHash = '';


        if (!isset($receivedData->subscriptionId) || empty($receivedData->subscriptionId)) {

            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter subscription_id";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
            // $this->sendResponse("Failed", $message, $error);
        }

        if (!isset($receivedData->customerHash) || empty($receivedData->customerHash)) {

            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter recharge customer hash";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
            // $this->sendResponse("Failed", $message, $error);
        }

        $subscriptionId = trim($receivedData->subscriptionId);
        $customerHash = trim($receivedData->customerHash);

        $subscriptionDetails = $this->rechargeObj->getSubscription($subscriptionId);

        if (!isset($subscriptionDetails) || isset($subscriptionDetails->error)
         || isset($subscriptionDetails->errors) || empty($subscriptionDetails->subscription->customer_id)) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($subscriptionDetails->error)) {

                $error = $subscriptionDetails->error;

            } else if (isset($subscriptionDetails->errors)) {

                $error = $subscriptionDetails->errors;
            }
            $message = "Failed to get the subscription details";

            $this->helperObj->logMe(
                "Failed to get the subscription details: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error,
                $this->logFiles['subscriptionReactivationWithCoupon']['filename']
            );

            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
            // $this->sendResponse("Failed", $message, $error);
        }

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($subscriptionDetails->subscription->customer_id);

        if (!isset($customerDetails) || isset($customerDetails->error)
         || isset($customerDetails->errors) || empty($customerDetails->customer->hash)) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($customerDetails->error)) {

                $error = $customerDetails->error;

            } else if (isset($customerDetails->errors)) {

                $error = $customerDetails->errors;
            }
            $message = "Failed to get the customer details from recharge";

            $this->helperObj->logMe(
                "Failed to get the customer details from recharge: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error,
                $this->logFiles['subscriptionReactivationWithCoupon']['filename']
            );

            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
            // $this->sendResponse("Failed", $message, $error);
        }

        $rechargeCustomerHash = trim($customerDetails->customer->hash);

        if ($customerHash == $rechargeCustomerHash) {

            $reactivateResponse = $this->rechargeObj->activateSubscription($subscriptionId);

            if (isset($reactivateResponse->error) || isset($reactivateResponse->errors) || empty($reactivateResponse)) {

                $error = "Failed subscriptions - ".$subscriptionId.' - '.$reactivateResponse->error;
                $message = "Failed to reactivate the subscription";
                $status = "Failed";

                $this->helperObj->logMe(
                    "Failed to reactivate the subscription: ".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId.
                    " | Error -> ".$error,
                    $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                );

                $response = array(
                    "status"       => $status,
                    "message" => $message,
                    "error" => $error
                );
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
                // $this->sendResponse("Failed", $message, $error);
            }
            else{


                $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionId);

                if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
                    empty($subscriptionChargeDetails)) {

                    $status = "Failed";
                    $error  = "Unknown error";
                    if (!empty($subscriptionChargeDetails->error)) {

                        $error = $subscriptionChargeDetails->error;

                    } else if (!empty($subscriptionChargeDetails->errors)) {

                        $error = "Failed to get the charge details by using subscriptionId: ".$subscriptionId;
                    }

                    $message = "Failed to get the subscription charge details from recharge";
                    $this->helperObj->logMe(
                        "Failed to get the charge details by using subscriptionId:".
                        " | customerHash -> ".$customerHash.
                        " | Subscription -> ".$subscriptionId.
                        " | Error -> ".$error,
                        $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                    );

                    $response = array(
                        "status"       => $status,
                        "message" => $message,
                        "error" => $error
                    );
                    echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                    die();
                    // $this->sendResponse("Failed", $message, $error);
                }

                $subscriptionChargeStatus = null;
                $subscriptionChargeID = null;
                $queuedCharge = [];
                foreach ($subscriptionChargeDetails->charges as $chargeValue) {

                    $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
                    if (strpos($subscriptionChargeTags, 'subscription' ) !== false &&
                    strtolower(trim($chargeValue->status)) == 'queued' ) {

                        $subscriptionChargeID = $chargeValue->id;
                        $subscriptionChargeStatus = $chargeValue->status;
                        $queuedCharge = $chargeValue;
                    }
                }

                /* Iterable event for reactivating the subscription*/
                $subscribedItems = '';

                if(!empty($queuedCharge)){
                    $subscribedItems = $queuedCharge->line_items;
                }

                if(isset($customerDetails->customer->email) && !empty($customerDetails->customer->email)){

                    $lineItemDetails = [];
                    if(!empty($subscribedItems)){
                        foreach($subscribedItems as $item) {

                            $lineItem = new \stdClass();

                            $lineItem->variant_id = $item->shopify_variant_id;
                            $lineItem->quantity = $item->quantity;
                            $lineItem->sku = $item->sku;
                            $lineItem->price = $item->price;
                            $lineItem->title = $item->title;
                            $lineItemDetails[] = $lineItem;

                        }
                    }

                    // print_r($lineItemDetails);die;

                    $customerEmail = $customerDetails->customer->email;

                    $eventRequest = array(
                        "events" => array(
                            array("email" => strtolower($customerEmail),
                                "eventName" => "subscriptionReactivated",
                                "dataFields" => array("lineItems" => $lineItemDetails)
                            )
                        )
                    );
                    $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

                    if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {
                        $this->helperObj->logMe(
                            "Subscription reactivation event - Successfully created for Email (".
                            strtolower($customerEmail).")".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );
                    } else {
                        $this->helperObj->logMe(
                            "Subscription reactivation event - Failed to create event for Email (".
                            strtolower($customerEmail).")".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );
                    }
                }
                /* Iterable event for reactivating the subscription*/

                /* apply coupon for next charge */
                if (isset($receivedData->couponCode) && !empty($receivedData->couponCode)) {

                    if (empty($subscriptionChargeStatus)) {

                        $message = "The subscription queued charge not found for apply the discount subscriptionID: "
                        .$subscriptionId;
                        $error = "Queued charge not found";
                        $status = "Failed";

                        $this->helperObj->logMe(
                            "Queued charge not found:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );

                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();

                        //$this->sendResponse($status, $message, $error);
                    }


                    $discountCode = trim($receivedData->couponCode);
                    $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
                    if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
                    empty($discountCodeDetails)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountCodeDetails->error)) {

                            $error = $discountCodeDetails->error;

                        } else if (!empty($discountCodeDetails->errors)) {

                            $error = "Failed to get the discount code details from recharge";
                        }

                        $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                        .$discountCode;
                        $this->helperObj->logMe(
                            "Failed to get the discount details from recharge by using the provided discountCode:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | DiscountCode -> ".$discountCode.
                            " Error -> ".$error,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );

                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();
                        //$this->sendResponse($status, $message, $error);
                    }
                    $discountRequest = array(
                        "discount_code" => $discountCode
                    );

                    $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionChargeID,
                    json_encode($discountRequest));

                    if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
                    empty($discountApplyResponse)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountApplyResponse->error)) {

                            $error = $discountApplyResponse->error;

                        } else if (!empty($discountApplyResponse->errors)) {

                            $error = "Failed to apply the discount to the charge";
                        }

                        $message = "Failed to apply the discount to the charge chargeID: ".$subscriptionChargeID;
                        $this->helperObj->logMe(
                            "Failed to apply the discount to the charge: ".
                            " | Subscription -> ".$subscriptionId.
                            " | customerHash -> ".$customerHash.
                            " | chargeID: ".$subscriptionChargeID.
                            " | DiscountCode: ".$discountCode.
                            " | Error -> ".json_encode($error),
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );
                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();
                        // $this->sendResponse($status, $message, $error);
                    }


                    $message = "Successfully created the subscription and applied the discount";
                    $error = "";
                    $status = "Success";

                    $this->helperObj->logMe(
                        "Successfully created the subscription and applied the discount.".
                        " | Subscription -> ".$subscriptionId.
                        " | customerHash -> ".$customerHash.
                        " | chargeID: ".$subscriptionChargeID.
                        " | DiscountCode: ".$discountCode,
                        $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                    );

                    $response = array(
                        "status"  => $status,
                        "message" => $message,
                        "error"   => $error
                    );
                    echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                    die();

                    // $this->sendResponse($status, $message, $error);
                }
                /* apply coupon for next charge */

                $this->helperObj->logMe(
                    "Subscription reactivated successfully.".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId,
                    $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                );

                $message = "Subscription reactivated successfully";
                $error = "";
                $status = "Success";
                $response = array(
                        "status"  => $status,
                        "message" => $message,
                        "error"   => $error
                );
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
                // $this->sendResponse("Success", "Subscription reactivated successfully", "");
            }

        } else {
            $message = "Failed to reactivate the subscription";
            $error = "Customer hash tags are not matched";
            $status = "Failed";
            $this->helperObj->logMe(
                "Failed to reactivate the subscription. Customer hash tags are not matched:".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId,
                $this->logFiles['subscriptionReactivationWithCoupon']['filename']
            );
            $response = array(
                "status"  => $status,
                "message" => $message,
                "error"   => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
            //$this->sendResponse("Failed", $message, $error);
        }
    }
    
    
    /**
     * Convert OTP to subscription
     */
    public function otpToSubscription(request $data) {

        if (empty($data['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $receivedData = json_decode($data['data']);
        if (empty($receivedData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $emptyKey = null;
        $requiredRequestKeys = ["address_id", "hash_value"];
        $requestKeys = [];
        $valueStatus = 0;
        foreach ($receivedData as $key => $receiveValue) {

            $requestKeys[] = $key;
            if (empty($receiveValue) && $valueStatus == 0) {
                $emptyKey = $key;
                $valueStatus = 1;
            }
        }
        $keyCompareStatus = array_diff($requiredRequestKeys, $requestKeys);
        if(!empty($keyCompareStatus)) {

            $missedParameters = implode(",",$keyCompareStatus);
            $message = $missedParameters." endpoint parameters are missing";
            $error = "Some parameters are missing in the endpoint request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        if (!empty($emptyKey)) {

            $message = $emptyKey." is missing";
            $error = "Bad request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        
        $discountCode = null;
        $addressID = trim($receivedData->address_id);
        $otpChargeDetails = $this->rechargeObj->getOtpChargeDetails($addressID);
        if (!empty($otpChargeDetails->error) || !empty($otpChargeDetails->errors) ||
            empty($otpChargeDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($otpChargeDetails->error)) {

                $error = $otpChargeDetails->error;

            } else if (!empty($otpChargeDetails->errors)) {

                $error = "Failed to get the charge details by using addressID:".$addressID;
            }

            $message = "Failed to get the charge details from recharge";
            $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
            $this->requestResponse("Failed", $message, $error, $data->callback);
        }
        $customerHash = strtolower(trim($receivedData->hash_value));
        $originalCustomerHash = null;
        $customerEmail = null;
        $username = null;
        $rechargeChargeDetails = null;
        $subscriptionExistingStatus = null;
        foreach ($otpChargeDetails->charges as $chargeValue) {

            if ($addressID == trim($chargeValue->address_id) &&
            $customerHash == strtolower(trim($chargeValue->customer_hash))) {

                $originalCustomerHash = $chargeValue->customer_hash;
                $customerEmail = $chargeValue->email;
                $username  = !empty($chargeValue->first_name) ? $chargeValue->first_name.' ' : '';
                $username .= !empty($chargeValue->last_name) ? $chargeValue->last_name : '';
                $chargeTags = strtolower(trim($chargeValue->tags));
                if (strpos($chargeTags, 'otp' ) !== false) {
                    $rechargeChargeDetails = $chargeValue;
                }
                if (strpos($chargeTags, 'subscription' ) !== false &&
                strtolower(trim($chargeValue->status)) == 'queued' ) {
                    $subscriptionExistingStatus = $chargeValue->status;
                }
            }
        }
        
        if (empty($originalCustomerHash)) {

            $message = "Customer hash value is not matching";
            $error = "Security breach";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        if (!empty($subscriptionExistingStatus)) {

            $message = "There is already a queued charge in the given addressID:".$addressID;
            $error = "Existing an active queued charge";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $chargeTags = strtolower(trim($otpChargeDetails->charges[0]->tags));
        if (empty($rechargeChargeDetails)) {
            $message = "The charge in the given addressID is not an OTP charge";
            $error = "OTP charge not found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $otpLineItemQuantity = [];
        $lineItemProperties = [];
        $chargeVariantDetails = [];

        // $i = 0;
        $shopifyVariantId = '';
        $shopifyVariantQuantity = '';
        $shopifyVariantPrice = '';
        $shopifyVariantSku = '';
        $shopifyProductId = '';
        foreach ($rechargeChargeDetails->line_items as $item) {
            /*if (!empty($item)) {
               $chargeVariantDetails[$i]['variant_id'] = $item->shopify_variant_id;
               $chargeVariantDetails[$i]['quantity'] = $item->quantity;
               $i++;
            }*/
            $shopifyVariantId = $item->shopify_variant_id;
            $shopifyVariantQuantity = $item->quantity;
            $shopifyVariantPrice = $item->price;
            $shopifyVariantSku      = $item->sku;
            $shopifyProductId = $item->shopify_product_id;
        }
        if (empty($shopifyVariantId)) {

            $message = "The line items in the OTP charge is empty addressID:".$addressID;
            $error = "Line items not found in the OTP charge";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        
        $lineItemProperties[] = (object)array(
            "name" => "SKU",
            "value" => $shopifyVariantSku
        );

        $lineItemProperties[] = (object)array(
            "name" => "charge_interval_frequency",
            "value" => 27
        );
        $lineItemProperties[] = (object)array(
            "name" => "shipping_interval_frequency",
            "value" => 27
        );
        $lineItemProperties[] = (object)array(
            "name" => "shipping_interval_unit_type",
            "value" => "day"
        );
        

        $variantErrorStatus = 0;
        $subscriptionPrice = null;
        
        $variantAPiResponse = $this->api->rest('GET', '/admin/variants/'.$shopifyVariantId.'.json');
        
        if ( !empty($variantAPiResponse['errors']) || !empty($variantAPiResponse['error'])) {
            
            $variantError = !empty($variantAPiResponse['body']) ? json_encode($variantAPiResponse['body']) :
            json_encode($variantAPiResponse['status']);
            
            $message = "The variantID in the endpoint request is not valid variantID: ".$shopifyVariantId.", Error: ".$variantError;
            $error = "Invalid variantID found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $variantDetails = $variantAPiResponse['body'];
        
        $subscriptionPrice = floatval($subscriptionPrice) +
                    (floatval($variantDetails['variant']['price']) * intval($shopifyVariantQuantity));
        
        
        // $subscriptionPrice = floatval($subscriptionPrice) + (floatval($shopifyVariantPrice) * intval($shopifyVariantQuantity));

        if ($variantErrorStatus == 1) {

            $message = "The variantID in the endpoint request is not valid variantID: ".$variantMismatchID.", Error: ".$variantError;
            $error = "Invalid variantID found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $subscriptionSKU = "vitamin-pack";
        $today = time();
        $todayDate = date('Y-m-d');
        $defaultSubscriptionPeriod = 27;
        $otpChargeProcessedDate = date('Y-m-d', strtotime($rechargeChargeDetails->processed_at));
        $otpChargeProcessedDate = strtotime($otpChargeProcessedDate);
        $dateDiff = $today - $otpChargeProcessedDate;
        $dateDiffDays = round($dateDiff / (60 * 60 * 24));
        $nextChargePeriod = $defaultSubscriptionPeriod - $dateDiffDays;
        if ($nextChargePeriod <= 0) {
            $nextChargeDate = $todayDate;
        } else {
            $nextChargeDate = date('Y-m-d',strtotime('+'.$nextChargePeriod.' days',strtotime($todayDate)));
        }
        $intervalUnit = "day";

        $subscriptionRequest = array(
            "address_id" => $rechargeChargeDetails->address_id,
            "next_charge_scheduled_at" => $nextChargeDate,
            //"next_charge_scheduled_at" => $todayDate,
            "quantity" => !empty($rechargeChargeDetails->line_items[0]->quantity) ?
            $rechargeChargeDetails->line_items[0]->quantity : 1,
            "price" => floatval($subscriptionPrice),
            //"sku" => $subscriptionSKU,
            "shopify_variant_id" => $shopifyVariantId,
            "order_interval_unit" => $intervalUnit,
            "order_interval_frequency" => $defaultSubscriptionPeriod,
            "charge_interval_frequency" => $defaultSubscriptionPeriod,
            "properties" => $lineItemProperties
        );
        // echo '<pre>';
        // print_r($subscriptionRequest);die('-------------');

        $subscriptionCreateResponse = $this->rechargeObj->createSubscription(json_encode($subscriptionRequest));
        if (!empty($subscriptionCreateResponse->error) || !empty($subscriptionCreateResponse->errors) ||
            empty($subscriptionCreateResponse)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionCreateResponse->error)) {

                $error = $subscriptionCreateResponse->error;

            } else if (!empty($subscriptionCreateResponse->errors)) {

                $error = "Failed to create the subscription under the addressID:".$addressID;
            }
            $message = "Failed to create the subscription in recharge";
            $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
            $this->requestResponse("Failed", $message, $error, $data->callback);
        }
        // Create iterable event convertToSubscription
        $eventRequest = array(
            "events" => array(
            array("email" => strtolower($customerEmail),
            "eventName" => "convertToSubscription",
            "dataFields" => array("userName" => $username))
            )
        );
        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

            $this->helperObj->logMe("convertToSubscription iterable event - Successfully created event for Email (".
            strtolower($customerEmail).")", $this->logFiles['otpToSubscriptionLog']['filename']);
        } else {
            $this->helperObj->logMe("convertToSubscription iterable event - Failed to create event for Email (".
            strtolower($customerEmail).")", $this->logFiles['otpToSubscriptionLog']['filename']);
        }
        // Applying the discount to the created subscription charge
        $subscriptionID = $subscriptionCreateResponse->subscription->id;
        $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionID);
        if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
            empty($subscriptionChargeDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionChargeDetails->error)) {

                $error = $subscriptionChargeDetails->error;

            } else if (!empty($subscriptionChargeDetails->errors)) {

                $error = "Failed to get the charge details by using subscriptionID: ".$subscriptionID;
            }

            $message = "Failed to get the subscription charge details from recharge";
            $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
            $this->requestResponse("Failed", $message, $error, $data->callback);
        }
        $subscriptionChargeStatus = null;
        $subscriptionchargeID = null;
        foreach ($subscriptionChargeDetails->charges as $chargeValue) {

            $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
            if (strpos($subscriptionChargeTags, 'subscription' ) !== false &&
            strtolower(trim($chargeValue->status)) == 'queued' ) {

                $subscriptionchargeID = $chargeValue->id;
                $subscriptionChargeStatus = $chargeValue->status;
            }
        }
        if (empty($subscriptionChargeStatus)) {

            $message = "The subscription queued charge not found for apply the discount subscriptionID: "
            .$subscriptionID;
            $error = "Queued charge not found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        if (!empty($receivedData->discountCode)) {

            $discountCode = trim($receivedData->discountCode);
            $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
            if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
            empty($discountCodeDetails)) {

                $status = "failed";
                $error  = "Unknown error";
                if (!empty($discountCodeDetails->error)) {

                    $error = $discountCodeDetails->error;

                } else if (!empty($discountCodeDetails->errors)) {

                    $error = "Failed to get the discount code details from recharge";
                }

                $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                .$discountCode;
                $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
                $this->requestResponse("Failed", $message, $error, $data->callback);
            }
            $discountRequest = array(
                "discount_code" => $discountCode
            );
            $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionchargeID,
            json_encode($discountRequest));
            if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
            empty($discountApplyResponse)) {

                $status = "failed";
                $error  = "Unknown error";
                if (!empty($discountApplyResponse->error)) {

                    $error = $discountApplyResponse->error;

                } else if (!empty($discountApplyResponse->errors)) {

                    $error = "Failed to apply the discount to the charge";
                }

                $message = "Failed to apply the discount to the charge chargeID: ".$subscriptionchargeID;
                $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
                $this->requestResponse("Failed", $message, $error, $data->callback);
            }
            $message = "Successfully created the subscription and applied the discount";
            $error = "";
            $status = "Success";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $message = "Successfully created the subscription";
        $error = "";
        $status = "Success";
        $this->requestResponse($status, $message, $error, $data->callback);
    }
    
    public function requestResponse($status, $message, $error, $callback) {

        $response = array(
            "status"       => $status,
            "message" => $message,
            "error" => $error
        );
        echo $callback . '(' . json_encode($response) . ')';
        die();

    }



}
