<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Model\IterableApi;
use App\Helper\ResponseHelper;
use Log;
use App\Helper\Helper;
use App\Jobs\ShopifyCustomer;

class SurveyController extends Controller
{
    public $iterableObj;
    public $helper;
    
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;

    public function __construct()
    {
        $this->iterableObj = new IterableApi();
        $this->helper = new ResponseHelper();
        $this->helperObj = new Helper();
        
        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }
    
    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }
    
     public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');
        if (!is_dir($this->logPath)){

         //Directory does not exist, so lets create it.
         mkdir($this->logPath);
         chmod($this->logPath, 0777);

        }
        $this->logFiles = array(
            'log' => array(
                "type" => "text",
                "filename" => $this->logPath . "/prospect-log.txt",
                "mode" => ""),
            );
    }
    
    public function setWithLogFiles() {

        if(!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                   $handler = fopen($fileName, $fileDetails['mode']);
                   fputcsv($handler, $fileDetails['columns']);
                   fclose($handler);
                   touch($fileName);
                   chmod($fileName, 0777);
                }
            }
        }
   }

   /**
    * Update menopause quiz info to iterable
    */
    public function updateCustomer(Request $request) {
       
        $requestData = $request->input('data');
        if (empty($requestData)) {
 
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
 
        $requestData = json_decode($requestData);
        $email = !empty($requestData->email) ? trim($requestData->email) : '';
        $properties = !empty($requestData->properties) ? $requestData->properties : '';
 
        if (empty($email)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required email",
            );
 
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
 
        if (empty($properties)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required properties",
            );
 
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
 
        $assessment = '';
        $acceptsMarketing = false;
        $customProperties = [];
        
        foreach ($properties as $key => $value) {
            
            $customProperties[$key] = $value;
            if($key == 'assessment') {
                $assessment = $value;
            }
            if($key == 'acceptsMarketing' && trim(strtolower($value)) == 'yes') {
                $acceptsMarketing = true;
            }
            if(trim($key) == 'ageAtSignup' && !empty($value)) {
                $customProperties["userAge"] = (int)$value;
            }
        }
        $userIterableUpdateRequest = array(
            "email" => $email,
            "dataFields" => array(
                "quizAnswer" => $customProperties,
                "assessment" => $assessment,
                "accepts_marketing" => $acceptsMarketing
            )
        );
        
        $eventRequest = array(
                "events" => array(
                    array(
                        "email" => strtolower($email),
                        "eventName" => "quizCompletion"
                    )
                )
            );
    
        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));
 
        if (isset($eventResponse->successCount) && $eventResponse->successCount > 0) {
            Log::info("Quiz Completion event - Successfully created the event, Email: ".
            strtolower($email)."");
 
        } else {
            Log::info("Quiz Completion event - Failed to create event, Email: ".
            strtolower($email).", Error:".json_encode($eventResponse));
        }
        
        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));
 
        if (strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            
            Log::info("Successfully updated the customer properties to iterable, Email - ".$email);
 
            $response = array(
                "status" => "success",
                "message" => "Successfully updated the customer properties - ".$email,
                "errorMessage" => "",
            );
 
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
           
        } else {
            Log::info("Failed to update the customer properties - ".$email);
            Log::info(json_encode($userIterableUpdateResponse));
 
            $response         = array(
                "status" => "failed",
                "message" => "Failed to update the customer properties - ".$email,
                "errorMessage"  => json_encode($userIterableUpdateResponse),
            );
    
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
    }
    
    
    public function quizCompletionEvent(Request $request)
    {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        $requestData = json_decode($_REQUEST['data']);

        $email = $requestData->email;
        
        if (empty($email)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required email",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $eventRequest = array(
            "events" => array(
                array(
                    "email" => $email,
                    "eventName" => "quizCompletion"
                )
            )
        );

        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));
        Log::info("eventResponse");
        Log::info(json_encode($eventResponse));

        if (isset($eventResponse->successCount) || $eventResponse->successCount > 0) {
            Log::info("quizCompletion event - Successfully created event for Email (".
            strtolower($email).")");

            $response = array(
                "status" => "success",
                "message" => "quizCompletion event - Successfully created event  - ".$email,
                "errorMessage" => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        } else {
            Log::info("quizCompletion event - Failed to create event for Email (".
            strtolower($email).")");

            $response         = array(
                "status" => "failed",
                "message" => "quizCompletion event - Failed to create event - ".$email,
                "errorMessage"  => json_encode($eventResponse),
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }
    }

    public function sendProspectEvent(Request $request)
    {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        $requestData = json_decode($_REQUEST['data']);

        $email = $requestData->email;
        
        if (empty($email)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required email",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }
        
        $userIterableUpdateRequest = array(
            "email" => $email,
            "dataFields" => array(
                "accepts_marketing" => true
            )
        );
        
        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));
        
        if (isset($userIterableUpdateResponse->code) && strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            Log::info("Successfully updated the accept marketing properties - ".$email);
        }else{
            Log::info("Failed to update the accept marketing - ".$email);
            Log::info(json_encode($userIterableUpdateResponse));
        }

        $eventRequest = array(
            "events" => array(
                array(
                    "email" => $email,
                    "eventName" => "prospect"
                )
            )
        );

        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));
        Log::info("eventResponse");
        Log::info(json_encode($eventResponse));

        if (isset($eventResponse->successCount) || $eventResponse->successCount > 0) {
            Log::info("Prospect event - Successfully created event for Email (".
            strtolower($email).")");

            $response = array(
                "status" => "success",
                "message" => "Prospect event - Successfully created event  - ".$email,
                "errorMessage" => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        } else {
            Log::info("Prospect event - Failed to create event for Email (".
            strtolower($email).")");

            $response         = array(
                "status" => "failed",
                "message" => "Prospect event - Failed to create event - ".$email,
                "errorMessage"  => json_encode($eventResponse),
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

    }
    
    public function sendProspectNotification(Request $request)
    {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        $requestData = json_decode($_REQUEST['data']);

        $email = $requestData->email;
        
        if (empty($email)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required email",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }
        
        $this->helperObj->logMe("dispatched to create customer in shopify- ".$email, $this->logFiles['log']['filename']);
                        
    	$queueResponse = dispatch(new ShopifyCustomer($requestData, $this->logFiles['log']['filename']));
        

        $eventRequest = array(
            "events" => array(
                array(
                    "email" => $email,
                    "eventName" => "prospect"
                )
            )
        );
        die;
        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->successCount) && $eventResponse->successCount > 0) {
            Log::info("Prospect event - Successfully created event for Email (".
            strtolower($email).")");

            $response = array(
                "status" => "success",
                "message" => "Prospect event - Successfully created event  - ".$email,
                "errorMessage" => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        } else {
            Log::info("Prospect event - Failed to create event for Email (".
            strtolower($email).")");
            Log::info(json_encode($eventResponse));

            $response         = array(
                "status" => "failed",
                "message" => "Prospect event - Failed to create event - ".$email,
                "errorMessage"  => json_encode($eventResponse),
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

    }
}
