<?php

namespace App\Jobs;

use App\Model\IterableApi;
use App\Model\RechargeApi;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Log;
use App\Helper\Helper;

class IterableSubscriptionCancelEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $rechargeObj;
    private $customerID;
    private $properties;
    private $requestData;
    private $helperObj;
    private $logFile;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData, $logPath)
    {
        $this->requestData = $requestData;
        $this->customerID = $requestData;
        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();
        $this->helperObj   = new Helper();
        $this->logFile     = $logPath;
    }

    public function handle()
    {

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($this->customerID);
        
        $this->helperObj->logMe("Reached to subscription cancel job", $this->logFile);
        $this->helperObj->logMe("customerDetails".json_encode($customerDetails), $this->logFile);

        if (isset($customerDetails->error) || isset($customerDetails->errors) ||
            empty($customerDetails->customer->email) || empty($customerDetails)) {
            $status = "failed";
            $error = "Unknown error";

            if (isset($customerDetails->error)) {
                $error = $customerDetails->error;
            } elseif (isset($customerDetails->errors)) {
                $error = "Request is not correct or missing the required parameters";
            }
                
            $this->helperObj->logMe("Cancelled subscription iterable event - Failed to get the Recharge user details for CustomerID (" .
                $this->customerID . ")", $this->logFile);

            die();
        }

        $eventRequest = array(
            "events" => array(
                array(
                    "email" => strtolower($customerDetails->customer->email),
                    "eventName" => "subscriptionCancelled",
                    "dataFields" => array(
                        "userName" => $customerDetails->customer->first_name . ' ' . $customerDetails->customer->last_name,
                    ),
                ),
            ),
        );

        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));
        
        $this->helperObj->logMe("eventResponse".json_encode($eventResponse), $this->logFile);

        if (isset($eventResponse->successCount) && $eventResponse->successCount > 0) {
           
            $this->helperObj->logMe("Cancelled subscription iterable event - Successfully created event for Email (" .
                strtolower($customerDetails->customer->email) . ")".json_encode($eventResponse), $this->logFile);
        } else {
            
            $this->helperObj->logMe("Cancelled subscription iterable event - Failed to create event for Email (" .
                strtolower($customerDetails->customer->email) . ")".json_encode($eventResponse), $this->logFile);
        }
    }
}
